@@ -25,17 +25,20 @@ namespace torali
 
   struct CountDNAConfig {
     bool adaptive;
-    bool hasPanelFile;
     bool hasStatsFile;
     bool hasBedFile;
     bool hasScanFile;
     bool noScanWindowSelection;
+    bool segmentation;
+    bool hasGenoFile;
+    bool hasVcfFile;
     uint32_t nchr;
     uint32_t meanisize;
     uint32_t window_size;
     uint32_t window_offset;
     uint32_t scanWindow;
     uint32_t minChrLen;
+    uint32_t minCnvSize;
     uint16_t minQual;
     uint16_t mad;
     uint16_t ploidy;
@@ -44,9 +47,13 @@ namespace torali
     float fracWindow;
     float fragmentUnique;
     float controlMaf;
+    float stringency;
+    float cn_offset;
     std::string sampleName;
-    boost::filesystem::path outfile;
-    boost::filesystem::path panelfile;
+    boost::filesystem::path vcffile;
+    boost::filesystem::path genofile;
+    boost::filesystem::path cnvfile;
+    boost::filesystem::path covfile;
     boost::filesystem::path genome;
     boost::filesystem::path statsFile;
     boost::filesystem::path mapFile;
@@ -54,7 +61,7 @@ namespace torali
     boost::filesystem::path bedFile;
     boost::filesystem::path scanFile;
   };
-
+  
   struct CountDNAConfigLib {
     uint16_t madCutoff;
     uint16_t madNormalCutoff;
@@ -90,13 +97,25 @@ namespace torali
     // Open output files
     boost::iostreams::filtering_ostream dataOut;
     dataOut.push(boost::iostreams::gzip_compressor());
-    dataOut.push(boost::iostreams::file_sink(c.outfile.c_str(), std::ios_base::out | std::ios_base::binary));
+    dataOut.push(boost::iostreams::file_sink(c.covfile.c_str(), std::ios_base::out | std::ios_base::binary));
     dataOut << "chr\tstart\tend\t" << c.sampleName << "_mappable\t" << c.sampleName << "_counts\t" << c.sampleName << "_CN" << std::endl;
 
-    boost::iostreams::filtering_ostream panelOut;
-    if (c.hasPanelFile) {
-      panelOut.push(boost::iostreams::gzip_compressor());
-      panelOut.push(boost::iostreams::file_sink(c.panelfile.c_str(), std::ios_base::out | std::ios_base::binary));
+    // CNVs
+    std::vector<CNV> cnvs;
+    if (c.hasGenoFile) parseVcfCNV(c, hdr, cnvs);
+
+    // SVs for breakpoint refinement
+    typedef std::vector<SVBreakpoint> TChrBreakpoints;
+    typedef std::vector<TChrBreakpoints> TGenomicBreakpoints;
+    TGenomicBreakpoints svbp(c.nchr, TChrBreakpoints());
+    if (c.hasVcfFile) {
+      std::vector<StructuralVariantRecord> svs;
+      vcfParse(c, hdr, svs);
+      for(uint32_t i = 0; i < svs.size(); ++i) {
+	svbp[svs[i].chr].push_back(SVBreakpoint(svs[i].svStart, svs[i].ciposlow, svs[i].ciposhigh, svs[i].mapq));
+	svbp[svs[i].chr2].push_back(SVBreakpoint(svs[i].svEnd, svs[i].ciendlow, svs[i].ciendhigh, svs[i].mapq));
+      }
+      for (uint32_t i = 0; i < svbp.size(); ++i) sort(svbp[i].begin(), svbp[i].end(), SortSVBreakpoint<SVBreakpoint>());
     }
     
     // Iterate chromosomes
@@ -215,10 +234,21 @@ namespace torali
 	hts_itr_destroy(iter);
       }
 
+      // CNV discovery
+      if (!c.hasGenoFile) {
+	// Call CNVs
+	std::vector<CNV> chrcnv;
+	callCNVs(c, gcbound, gcContent, uniqContent, gcbias, cov, hdr, refIndex, chrcnv);
 
-      // Call CNVs
-      //callCNVs(c, gcbound, gcContent, uniqContent, gcbias, cov, hdr, refIndex);
+	// Merge adjacent CNVs lacking read-depth shift
+	mergeCNVs(c, chrcnv, cnvs);
 
+	// Refine breakpoints
+	if (c.hasVcfFile) breakpointRefinement(c, gcbound, gcContent, uniqContent, gcbias, cov, hdr, refIndex, svbp, cnvs);
+      }
+      
+      // CNV genotyping
+      genotypeCNVs(c, gcbound, gcContent, uniqContent, gcbias, cov, hdr, refIndex, cnvs);
 
       // BED File (target intervals)
       if (c.hasBedFile) {
@@ -259,7 +289,8 @@ namespace torali
 		    if (winlen == c.window_size) {
 		      obsexp /= (double) winlen;
 		      double count = ((double) covsum / obsexp ) * (double) c.window_size / (double) winlen;
-		      double cn = c.ploidy * covsum / expcov;
+		      double cn = c.ploidy;
+		      if (expcov > 0) cn = c.ploidy * covsum / expcov;
 		      dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (pos + 1) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 		      // reset
 		      covsum = 0;
@@ -314,7 +345,8 @@ namespace torali
 	      if (winlen >= c.fracWindow * (it->second - it->first)) {
 		obsexp /= (double) winlen;
 		double count = ((double) covsum / obsexp ) * (double) (it->second - it->first) / (double) winlen;
-		double cn = c.ploidy * covsum / expcov;
+		double cn = c.ploidy;
+		if (expcov > 0) cn = c.ploidy * covsum / expcov;
 		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << it->first << "\t" << it->second << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 	      } else {
 		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << it->first << "\t" << it->second << "\tNA\tNA\tNA" << std::endl;
@@ -340,7 +372,8 @@ namespace torali
 	      if (winlen == c.window_size) {
 		obsexp /= (double) winlen;
 		double count = ((double) covsum / obsexp ) * (double) c.window_size / (double) winlen;
-		double cn = c.ploidy * covsum / expcov;
+		double cn = c.ploidy;
+		if (expcov > 0) cn = c.ploidy * covsum / expcov;
 		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (pos + 1) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 		// reset
 		covsum = 0;
@@ -387,63 +420,22 @@ namespace torali
 	      if (winlen >= c.fracWindow * c.window_size) {
 		obsexp /= (double) winlen;
 		double count = ((double) covsum / obsexp ) * (double) c.window_size / (double) winlen;
-		double cn = c.ploidy * covsum / expcov;
+		double cn = c.ploidy;
+		if (expcov > 0) cn = c.ploidy * covsum / expcov;
 		dataOut << std::string(hdr->target_name[refIndex]) << "\t" << start << "\t" << (start + c.window_size) << "\t" << winlen << "\t" << count << "\t" << cn << std::endl;
 	      }
 	    }
 	  }
 	}
-
-	// Panel output?
-	if (c.hasPanelFile) {
-	  // cnv tiling windows
-	  uint32_t smallestWin = 300;
-	  uint32_t largestWin = 40000;
-	  uint32_t numTiling = 0;
-	  {
-	    uint32_t winbound = smallestWin;
-	    while (winbound < largestWin) {
-	      ++numTiling;
-	      winbound *= 2;
-	    }
-	  }
-	  std::vector<int32_t> cnvec(numTiling, -1);
-	  panelOut << "#" << std::string(hdr->target_name[refIndex]) << "," << hdr->target_len[refIndex] << "," << smallestWin << "," << largestWin << "," << c.sampleName << std::endl;
-	  for(uint32_t start = 0; start < hdr->target_len[refIndex]; start = start + 50) {
-	    if (start + smallestWin < hdr->target_len[refIndex]) {
-	      double covsum = 0;
-	      double expcov = 0;
-	      uint32_t winlen = 0;
-	      uint32_t winbound = smallestWin;
-	      uint32_t tilingPos = 0;
-	      std::fill(cnvec.begin(), cnvec.end(), -1);
-	      bool validCN = false;
-	      for(uint32_t pos = start; ((pos < start + largestWin) && (pos < hdr->target_len[refIndex])); ++pos) {
-		if ((gcContent[pos] > gcbound.first) && (gcContent[pos] < gcbound.second) && (uniqContent[pos] >= c.fragmentUnique * c.meanisize)) {
-		  covsum += cov[pos];
-		  expcov += gcbias[gcContent[pos]].coverage;
-		  ++winlen;
-		}
-		// Multiple of window size?
-		if ((pos - start) == winbound) {
-		  if (winlen >= c.fracWindow * (pos - start)) {
-		    cnvec[tilingPos] = (int32_t) boost::math::round(c.ploidy * covsum / expcov * 100.0);
-		    validCN = true;
-		  }
-		  winbound *= 2;
-		  ++tilingPos;
-		}
-	      }
-	      if (validCN) {
-		for(uint32_t i = 0; i < numTiling; ++i) panelOut << cnvec[i] << ',';
-		panelOut << std::endl;
-	      } else panelOut << "-2," << std::endl;
-	    }
-	  }
-	}
       }
     }
 
+    // Sort CNVs
+    sort(cnvs.begin(), cnvs.end(), SortCNVs<CNV>());
+
+    // Genotype CNVs
+    cnvVCF(c, cnvs);
+
     // clean-up
     fai_destroy(faiRef);
     fai_destroy(faiMap);
@@ -452,10 +444,6 @@ namespace torali
     sam_close(samfile);
     dataOut.pop();
     dataOut.pop();
-    if (c.hasPanelFile) {
-      panelOut.pop();
-      panelOut.pop();
-    }
     
     return 0;
   }
@@ -472,21 +460,30 @@ namespace torali
       ("quality,q", boost::program_options::value<uint16_t>(&c.minQual)->default_value(10), "min. mapping quality")
       ("mappability,m", boost::program_options::value<boost::filesystem::path>(&c.mapFile), "input mappability map")
       ("ploidy,y", boost::program_options::value<uint16_t>(&c.ploidy)->default_value(2), "baseline ploidy")
-      ("fragment,e", boost::program_options::value<float>(&c.fragmentUnique)->default_value(0.97), "min. fragment uniqueness [0,1]")
-      ("statsfile,s", boost::program_options::value<boost::filesystem::path>(&c.statsFile), "gzipped stats output file (optional)")
-      ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.outfile)->default_value("out.cov.gz"), "output file")
-      ("adaptive-windowing,a", "use mappable bases for window size")
+      ("outfile,o", boost::program_options::value<boost::filesystem::path>(&c.cnvfile)->default_value("cnv.bcf"), "output CNV file")
+      ("covfile,c", boost::program_options::value<boost::filesystem::path>(&c.covfile)->default_value("cov.gz"), "output coverage file")
       ;
 
-    boost::program_options::options_description window("Window options");
+    boost::program_options::options_description cnv("CNV calling");
+    cnv.add_options()
+      ("sdrd,x", boost::program_options::value<float>(&c.stringency)->default_value(2), "min. SD read-depth shift")
+      ("cn-offset,t", boost::program_options::value<float>(&c.cn_offset)->default_value(0.1), "min. CN offset")
+      ("cnv-size,z", boost::program_options::value<uint32_t>(&c.minCnvSize)->default_value(1000), "min. CNV size")
+      ("svfile,l", boost::program_options::value<boost::filesystem::path>(&c.vcffile), "delly SV file for breakpoint refinement")
+      ("vcffile,v", boost::program_options::value<boost::filesystem::path>(&c.genofile), "input VCF/BCF file for re-genotyping")
+      ("segmentation,u", "copy-number segmentation")
+      ;
+    
+    boost::program_options::options_description window("Read-depth windows");
     window.add_options()
       ("window-size,i", boost::program_options::value<uint32_t>(&c.window_size)->default_value(10000), "window size")
       ("window-offset,j", boost::program_options::value<uint32_t>(&c.window_offset)->default_value(10000), "window offset")
       ("bed-intervals,b", boost::program_options::value<boost::filesystem::path>(&c.bedFile), "input BED file")
       ("fraction-window,k", boost::program_options::value<float>(&c.fracWindow)->default_value(0.25), "min. callable window fraction [0,1]")
+      ("adaptive-windowing,a", "use mappable bases for window size")
       ;
 
-    boost::program_options::options_description gcopt("GC options");
+    boost::program_options::options_description gcopt("GC fragment normalization");
     gcopt.add_options()
       ("scan-window,w", boost::program_options::value<uint32_t>(&c.scanWindow)->default_value(10000), "scanning window size")
       ("fraction-unique,f", boost::program_options::value<float>(&c.uniqueToTotalCovRatio)->default_value(0.8), "uniqueness filter for scan windows [0,1]")
@@ -499,7 +496,8 @@ namespace torali
     boost::program_options::options_description hidden("Hidden options");
     hidden.add_options()
       ("input-file", boost::program_options::value<boost::filesystem::path>(&c.bamFile), "input bam file")
-      ("panelfile,z", boost::program_options::value<boost::filesystem::path>(&c.panelfile), "output panel file")
+      ("fragment,e", boost::program_options::value<float>(&c.fragmentUnique)->default_value(0.97), "min. fragment uniqueness [0,1]")
+      ("statsfile,s", boost::program_options::value<boost::filesystem::path>(&c.statsFile), "gzipped stats output file (optional)")
       ;
 
     boost::program_options::positional_options_description pos_args;
@@ -507,9 +505,9 @@ namespace torali
 
     // Set the visibility
     boost::program_options::options_description cmdline_options;
-    cmdline_options.add(generic).add(window).add(gcopt).add(hidden);
+    cmdline_options.add(generic).add(cnv).add(window).add(gcopt).add(hidden);
     boost::program_options::options_description visible_options;
-    visible_options.add(generic).add(window).add(gcopt);
+    visible_options.add(generic).add(cnv).add(window).add(gcopt);
 
     // Parse command-line
     boost::program_options::variables_map vm;
@@ -551,15 +549,57 @@ namespace torali
     if (vm.count("adaptive-windowing")) c.adaptive = true;
     else c.adaptive = false;
 
-    // CNV mode
-    if (vm.count("panelfile")) c.hasPanelFile = true;
-    else c.hasPanelFile = false;
-    
+    // Segmentation
+    if (vm.count("segmentation")) c.segmentation = true;
+    else c.segmentation = false;
+
     // Check window size
     if (c.window_offset > c.window_size) c.window_offset = c.window_size;
     if (c.window_size == 0) c.window_size = 1;
     if (c.window_offset == 0) c.window_offset = 1;
 
+    // Check input VCF file (CNV genotyping)
+    if (vm.count("vcffile")) {
+      if (!(boost::filesystem::exists(c.genofile) && boost::filesystem::is_regular_file(c.genofile) && boost::filesystem::file_size(c.genofile))) {
+	std::cerr << "Input VCF/BCF file is missing: " << c.genofile.string() << std::endl;
+	return 1;
+      }
+      htsFile* ifile = bcf_open(c.genofile.string().c_str(), "r");
+      if (ifile == NULL) {
+	std::cerr << "Fail to open file " << c.genofile.string() << std::endl;
+	return 1;
+      }
+      bcf_hdr_t* hdr = bcf_hdr_read(ifile);
+      if (hdr == NULL) {
+	std::cerr << "Fail to open index file " << c.genofile.string() << std::endl;
+	return 1;
+      }
+      bcf_hdr_destroy(hdr);
+      bcf_close(ifile);
+      c.hasGenoFile = true;
+    } else c.hasGenoFile = false;
+
+    // Check input VCF file (delly SV file)
+    if (vm.count("svfile")) {
+      if (!(boost::filesystem::exists(c.vcffile) && boost::filesystem::is_regular_file(c.vcffile) && boost::filesystem::file_size(c.vcffile))) {
+	std::cerr << "Input VCF/BCF file is missing: " << c.vcffile.string() << std::endl;
+	return 1;
+      }
+      htsFile* ifile = bcf_open(c.vcffile.string().c_str(), "r");
+      if (ifile == NULL) {
+	std::cerr << "Fail to open file " << c.vcffile.string() << std::endl;
+	return 1;
+      }
+      bcf_hdr_t* hdr = bcf_hdr_read(ifile);
+      if (hdr == NULL) {
+	std::cerr << "Fail to open index file " << c.vcffile.string() << std::endl;
+	return 1;
+      }
+      bcf_hdr_destroy(hdr);
+      bcf_close(ifile);
+      c.hasVcfFile = true;
+    } else c.hasVcfFile = false;
+    
     // Check bam file
     LibraryInfo li;
     if (!(boost::filesystem::exists(c.bamFile) && boost::filesystem::is_regular_file(c.bamFile) && boost::filesystem::file_size(c.bamFile))) {
