@@ -176,6 +176,8 @@ namespace torali
 	} else {
 	  // SR support and qualities
 	  std::sort(qualStore[svid].begin(), qualStore[svid].end());
+	  svs[svid].mapq = 0;
+	  for(uint32_t i = 0; i < qualStore[svid].size(); ++i) svs[svid].mapq += qualStore[svid][i];
 	  svs[svid].srSupport = seqStore[svid].size();
 	  svs[svid].srMapQuality = qualStore[svid][qualStore[svid].size()/2];
 	}
@@ -221,6 +223,8 @@ namespace torali
 	  } else {
 	    // SR support and qualities
 	    std::sort(traQualStore[svid].begin(), traQualStore[svid].end());
+	    svs[svid].mapq = 0;
+	    for(uint32_t i = 0; i < traQualStore[svid].size(); ++i) svs[svid].mapq += traQualStore[svid][i];
 	    svs[svid].srSupport = traStore[svid].size();
 	    svs[svid].srMapQuality = traQualStore[svid][traQualStore[svid].size()/2];
 	  }
@@ -324,24 +328,23 @@ namespace torali
 	    uint32_t* cigar = bam_get_cigar(rec);
 	    for (std::size_t i = 0; i < rec->core.n_cigar; ++i) {
 	      if ((bam_cigar_op(cigar[i]) == BAM_CMATCH) || (bam_cigar_op(cigar[i]) == BAM_CEQUAL) || (bam_cigar_op(cigar[i]) == BAM_CDIFF)) {
-		// match or mismatch
-		for(std::size_t k = 0; k<bam_cigar_oplen(cigar[i]);++k) {
-		  ++sp;
-		  ++rp;
-		}
+		sp += bam_cigar_oplen(cigar[i]);
+		rp += bam_cigar_oplen(cigar[i]);
 	      } else if (bam_cigar_op(cigar[i]) == BAM_CDEL) {
 		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, false);
 		rp += bam_cigar_oplen(cigar[i]);
 		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, true);
 	      } else if (bam_cigar_op(cigar[i]) == BAM_CINS) {
+		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, false);
 		sp += bam_cigar_oplen(cigar[i]);
+		if (bam_cigar_oplen(cigar[i]) > c.minRefSep) _insertJunction(readBp, seed, rec, rp, sp, true);
 	      } else if ((bam_cigar_op(cigar[i]) == BAM_CSOFT_CLIP) || (bam_cigar_op(cigar[i]) == BAM_CHARD_CLIP)) {
 		int32_t finalsp = sp;
 		bool scleft = false;
 		if (sp == 0) {
 		  finalsp += bam_cigar_oplen(cigar[i]); // Leading soft-clip / hard-clip
 		  scleft = true;
-		  }
+		}
 		sp += bam_cigar_oplen(cigar[i]);
 		if (bam_cigar_oplen(cigar[i]) > c.minClip) _insertJunction(readBp, seed, rec, rp, finalsp, scleft);
 	      } else if (bam_cigar_op(cigar[i]) == BAM_CREF_SKIP) {
@@ -450,10 +453,10 @@ namespace torali
 
       // Cluster
       cluster(c, srBR[svt], srSVs, c.maxReadSep, svt);
-    }
 
-    // Debug SR SVs
-    //outputStructuralVariants(c, srSVs);
+      // Debug SR SVs
+      //outputStructuralVariants(c, srSVs, svt);
+    }
 
     // Cluster paired-end records
     now = boost::posix_time::second_clock::local_time();
@@ -538,6 +541,7 @@ namespace torali
 	      itOther->srAlignQuality = sr[i].srAlignQuality;
 	      itOther->precise = true;
 	      itOther->consensus = sr[i].consensus;
+	      itOther->mapq += sr[i].mapq;
 	    }
 	  }
 	}
@@ -573,7 +577,10 @@ namespace torali
 	      }
 	    }
 	  }
-	  if (!preciseDuplicate) pe.push_back(sr[i]);
+	  if (!preciseDuplicate) {
+	    pe.push_back(sr[i]);
+	    sort(pe.begin(), pe.end(), SortSVs<StructuralVariantRecord>());
+	  }
 	}
       }
     }
