@@ -19,6 +19,10 @@ namespace torali
   #ifndef LAST_BIN
   #define LAST_BIN 65535
   #endif
+
+  #ifndef MAX_CN
+  #define MAX_CN 10
+  #endif
   
   struct LibraryInfo {
     int32_t rs;
@@ -33,6 +37,34 @@ namespace torali
     LibraryInfo() : rs(0), median(0), mad(0), minNormalISize(0), minISizeCutoff(0), maxNormalISize(0), maxISizeCutoff(0), abnormal_pairs(0) {}
   };
 
+  struct CNV {
+    int32_t chr;
+    int32_t start;
+    int32_t end;
+    int32_t ciposlow;
+    int32_t ciposhigh;
+    int32_t ciendlow;
+    int32_t ciendhigh;
+    int32_t qval;
+    double cn;
+    double mappable;
+    double sd;
+
+
+    CNV() : chr(0), start(0), end(0), ciposlow(0), ciposhigh(0), ciendlow(0), ciendhigh(0), qval(0), cn(-1), mappable(0), sd(1) {}
+    CNV(int32_t const c, int32_t const s, int32_t const e, int32_t const cil, int32_t const cih, int32_t const cel, int32_t ceh, double const estcn, double const mp) : chr(c), start(s), end(e), ciposlow(cil), ciposhigh(cih), ciendlow(cel), ciendhigh(ceh), qval(0), cn(estcn), mappable(mp), sd(1) {}
+  };
+
+  template<typename TCNV>
+  struct SortCNVs : public std::binary_function<TCNV, TCNV, bool>
+  {
+    inline bool operator()(TCNV const& sv1, TCNV const& sv2) {
+      return ((sv1.chr<sv2.chr) || ((sv1.chr==sv2.chr) && (sv1.start<sv2.start)) || ((sv1.chr==sv2.chr) && (sv1.start==sv2.start) && (sv1.end<sv2.end)) || ((sv1.chr==sv2.chr) && (sv1.start==sv2.start) && (sv1.end==sv2.end) && (sv1.cn < sv2.cn)));
+    }
+  };
+
+
+
 
   // Read count struct
   struct ReadCount {
@@ -72,6 +104,8 @@ namespace torali
       else if (value=="3to5") return DELLY_SVT_TRANS + 2;
       else if (value=="5to3") return DELLY_SVT_TRANS + 3;
       else return -1;
+    } else if (svt == "CNV") {
+      return 9;
     } else {
       if (value=="3to3") return 0;
       else if (value=="5to5") return 1;
@@ -90,6 +124,7 @@ namespace torali
     else if (svt == 2) return "DEL";
     else if (svt == 3) return "DUP";
     else if (svt == 4) return "INS";
+    else if (svt == 9) return "CNV";
     else return "BND";
   }
 
@@ -130,7 +165,7 @@ namespace torali
 
   // Output directory/file checks
   inline bool
-  _outfileValid(boost::filesystem::path& outfile) {
+  _outfileValid(boost::filesystem::path const& outfile) {
     try {
       boost::filesystem::path outdir;
       if (outfile.has_parent_path()) outdir = outfile.parent_path();
